const vueDocs = require('vue-docgen-api');
const path = require('path');
const _ = require('lodash');
const fs = require('fs');
var async = require('async');
var genShared = require('./gen-shared');
const docFirstProps = [
  'title',
  'description',
  'icon',
  'group',
  'type',
  'inputType',
  'default',
  'hideExpression',
  'dataSource',
  'designFirst',
  'extends'
];
function tag(obj, tagName) {
  return obj.tags && obj.tags[tagName] && obj.tags[tagName][0].description;
}
function run(components, srcFolder, inspect) {
  let schemas = {};
  let enums = {};
  console.log('正在生成组件schema文件...');
  async.forEachOf(
    components,
    (filePath, name, callback) => {
      vueDocs.parse(filePath).then((doc) => {
        if (inspect) {
          console.log(`${name}的原始定义: `);
          console.log(JSON.stringify(doc, null, 2));
        }
        let schema = {
          $schema: '',
          description: doc.description,
          title: tag(doc, 'title'),
          group: tag(doc, 'group'),
          icon: tag(doc, 'icon'),
          extends: 'ToggleableItem'
        };
        let properties = {};
        _.each(doc.props, (p) => {
          if (tag(p, 'ignore')) {
            return;
          }
          let _p = {
            type: null,
            title: tag(p, 'title'),
            description: null
          };
          let defaultValue = p.defaultValue && p.defaultValue.value;
          if (_.startsWith(defaultValue, "'")) {
            _p.default = _.trim(defaultValue, "'");
          } else if (defaultValue === 'false' || defaultValue === 'true') {
            _p.default = defaultValue === 'false' ? false : true;
          } else if (new RegExp(/\d+/).test(defaultValue)) {
            _p.default = _.toNumber(defaultValue);
          }
          if (p.description) {
            _p.description = p.description;
          }
          let type = p.type && p.type.name;
          if (type) {
            //复合类型取第一个类型
            let verticalLineIndex = type.indexOf('|');
            if (verticalLineIndex > 0) {
              type = type.substring(0, verticalLineIndex);
            }
            if (type === 'array') {
              _p.type = 'object[]';
            } else {
              _p.type = type;
            }
          } else {
            _p.type = 'string';
          }
          let tagDefinedType = tag(p, 'type');
          if (tagDefinedType) {
            _p.type = tagDefinedType;
          }
          //选项类型读取values构造枚举类型
          let jsonValues = tag(p, 'jsonValues');
          if (p.values || jsonValues) {
            let enumType = `${name}${_.upperFirst(p.name)}Values`;
            //@values red, blue, green
            let _values = null;
            if (p.values) {
              _values = p.values;
            } else if (jsonValues) {
              //@jsonValues [{"value": "small","title":"小"}, {"value": "medium","title":"中"}, {"value": "large","title":"大"}]
              //json数组格式的values需要转换为真实的数组格式
              _values = JSON.parse(jsonValues);
            }
            enums[enumType] = _values;
            _p.type = enumType;
          }
          let inputType = tag(p, 'inputType');
          if (inputType) {
            _p.inputType = inputType;
          }
          let dataSource = tag(p, 'dataSource');
          if (dataSource) {
            _p.dataSource = true;
          }
          let hideExpression = tag(p, 'hideExpression');
          if (hideExpression) {
            _p.hideExpression = true;
          }
          properties[p.name] = _p;
        });
        _.each(doc.slots, (_slot) => {
          let ignore = tag(_slot, 'ignore');
          if (ignore || !_slot.description) {
            return;
          }
          let _p = {
            type: 'object[]',
            slot: true,
            title: _slot.description
          };
          let designFirst = tag(_slot, 'designFirst');
          if (designFirst) {
            _p.designFirst = true;
          }
          let tagDefinedType = tag(_slot, 'type');
          if (tagDefinedType) {
            _p.type = tagDefinedType;
          }
          properties[_slot.name] = _p;
        });
        schema.properties = properties;
        schemas[name] = schema;
        // let events = _.map(doc.events, (evt) => {
        //   let evtName = '@' + evt.name;
        // });

        callback();
      });
    },
    (err) => {
      if (err) {
        return;
      }
      if (_.isEmpty(schemas)) {
        console.log('没有任何组件schema需要生成');
        return;
      }

      //生成共享schema方便各组件使用
      let sharedFolder = path.join(srcFolder, 'shared');
      genShared.run(sharedFolder);

      let elementsFolder = path.join(srcFolder, 'elements');
      //如果schema的源码目录elements目录不存在，先创建出来
      if (!fs.existsSync(elementsFolder)) {
        fs.mkdirSync(elementsFolder, { recursive: true });
      }
      _.forIn(schemas, (schema, name) => {
        schema.properties = schema.properties || {};
        let elementFile = path.join(elementsFolder, `${name}.json`);
        //读取已经存在的schema文件做合并
        if (fs.existsSync(elementFile)) {
          let oldElementSchema = fs.readFileSync(elementFile);
          oldElementSchema = JSON.parse(oldElementSchema);
          oldElementSchema.properties = oldElementSchema.properties || {};
          _.forIn(schema, (v, k) => {
            if (!_.isNil(v) && _.includes(docFirstProps, k)) {
              oldElementSchema[k] = v;
            }
          });
          _.forIn(schema.properties, (v, k) => {
            if (oldElementSchema.properties[k]) {
              _.forIn(v, (v2, k2) => {
                if (!_.isNil(v2) && _.includes(docFirstProps, k2)) {
                  oldElementSchema.properties[k][k2] = v2;
                }
              });
            } else {
              oldElementSchema.properties[k] = v;
            }
          });
          schema = oldElementSchema;
        }
        fs.writeFileSync(elementFile, JSON.stringify(schema, null, 2));
      });
      if (_.isEmpty(enums)) {
        console.log('组件schema文件生成完毕');
        return;
      }
      let enumsFolder = path.join(srcFolder, 'enums');
      if (!fs.existsSync(enumsFolder)) {
        fs.mkdirSync(enumsFolder, { recursive: true });
      }
      _.forIn(enums, (values, name) => {
        let enumFile = path.join(enumsFolder, `${name}.json`);
        let enumContent = {
          $schema: '',
          classType: 'Enum'
        };
        //读取已经存在的enum文件做合并
        if (fs.existsSync(enumFile)) {
          let oldEnum = fs.readFileSync(enumFile);
          oldEnum = JSON.parse(oldEnum);
          oldEnum.values = values;
          enumContent = oldEnum;
        } else {
          enumContent.values = values;
        }
        fs.writeFileSync(enumFile, JSON.stringify(enumContent, null, 2));
      });
      console.log('组件schema文件生成完毕');
    }
  );
}

module.exports = {
  run
};
